clear all, close all, clc

% Create the arrays for 3 different Hamming codes
[G_1,H_1,std_array_1] = hamming_matrices(3);
[G_2,H_2,std_array_2] = hamming_matrices(4);
[G_3,H_3,std_array_3] = hamming_matrices(5);

% Get code parameters from the generator check matrices
n_1 = size(G_1,2);
k_1 = size(G_1,1);
rate_1 = k_1/n_1;

n_2 = size(G_2,2);
k_2 = size(G_2,1);
rate_2 = k_2/n_2;

n_3 = size(G_3,2);
k_3 = size(G_3,1);
rate_3 = k_3/n_3;

% Define the SNR range
SNRdb = -2:2:12;

% Define the number of Monte-Carlo loops
loops = 100;

% Initialize the BER matrices
uncoded_bpsk_ber = zeros(length(SNRdb),loops);
coded_bpsk_ber_1 = zeros(length(SNRdb),loops);
coded_bpsk_ber_2 = zeros(length(SNRdb),loops);
coded_bpsk_ber_3 = zeros(length(SNRdb),loops);

for SNRloop = 1:length(SNRdb)
	
	SNRloop

	% Calculate the noise energy
	p_n = 1/10^(SNRdb(SNRloop)/10);
	p_n_coded_1 = (1/10^(SNRdb(SNRloop)/10))/rate_1;
	p_n_coded_2 = (1/10^(SNRdb(SNRloop)/10))/rate_2;
	p_n_coded_3 = (1/10^(SNRdb(SNRloop)/10))/rate_3;

	for l = 1:loops

		l

		% Uncoded BPSK
		x = randint(2^14,1);
		y = x;
		y(y == 0) = -1;
		r = y + sqrt(p_n)*randn(length(x),1);
		x_hat = (sign(r) == 1);
		uncoded_bpsk_ber(SNRloop,l) = mean(x_hat ~= x);

		% (7,4) Hamming coded BPSK (m = 3)
		x = randint(2^14,1);
		y = hamming_encoder(x, G_1);
		y(y == 0) = -1;
		r = y + sqrt(p_n_coded_1)*randn(1,length(y));
		r_hat = (sign(r) == 1);
		x_dec = hamming_decoder(r_hat, H_1, std_array_1);
		coded_bpsk_ber_1(SNRloop,l) = mean(x_dec ~= x');

		% (15,11) Hamming coded BPSK (m = 4)
		x = randint(11*2^12,1);
		y = hamming_encoder(x, G_2);
		y(y == 0) = -1;
		r = y + sqrt(p_n_coded_2)*randn(1,length(y));
		r_hat = (sign(r) == 1);
		x_dec = hamming_decoder(r_hat, H_2, std_array_2);
		coded_bpsk_ber_2(SNRloop,l) = mean(x_dec ~= x');
		
		% (31,26) Hamming coded BPSK (m = 5)
		x = randint(26*2^11,1);
		y = hamming_encoder(x, G_3);
		y(y == 0) = -1;
		r = y + sqrt(p_n_coded_3)*randn(1,length(y));
		r_hat = (sign(r) == 1);
		x_dec = hamming_decoder(r_hat, H_3, std_array_3);
		coded_bpsk_ber_3(SNRloop,l) = mean(x_dec ~= x');

	end
end

% Get mean values for BER
uncoded_bpsk_ber = mean(uncoded_bpsk_ber,2);
coded_bpsk_ber_1 = mean(coded_bpsk_ber_1,2);
coded_bpsk_ber_2 = mean(coded_bpsk_ber_2,2);
coded_bpsk_ber_3 = mean(coded_bpsk_ber_3,2);

% Plot
semilogy(SNRdb, uncoded_bpsk_ber,'black--')
hold on
semilogy(SNRdb, coded_bpsk_ber_1, 'blue')
semilogy(SNRdb, coded_bpsk_ber_2, 'red')
semilogy(SNRdb, coded_bpsk_ber_3, 'green')
ylabel('Bit Error Rate')
xlabel('SNR (dB)')
legend('Uncoded BPSK','(7,4) Hamming coded BPSK','(15,11) Hamming coded BPSK','(31,26) Hamming coded BPSK')
title('BER vs SNR for the AWGN channel')
hold off
