function x = hamming_encoder(y, H, std_array)
% Decodes the bitstream y using Hamming decoding with the given parity check matrix H
% and the coset leaders with their corresponding syndromes

	% Get code parameters from the parity check matrix
	n = size(H,2);
	m = size(H,1);
	k = n-m;

	% Split the std_array matrix into a matrix containing the coset leaders
	% and a matrix containing the corresponding syndromes
	coset_leaders = std_array(:,1:n);
	syndromes = std_array(:,n+1:end);	

	% Calculate the number of loops required to decode
	loops = ceil(length(y)/n);

	% Create a zero array
	zero_array = zeros(1,m);

	% Decode
	for p = 1:loops

		% Calculate the codeword syndrome
		syn = mod(y((p-1)*n+1:p*n)*H',2);
		
		% Recall that the generator matrix is in systematic form, so
		% the codeword consists of the actual information bits plus
		% some parity bits. The information bits are the k last bits 
		% of each codeword.

		% If the syndrome is zero, we don't need to correct any error
		if( syn == zero_array )
			x((p-1)*k+1:p*k) = y(p*m+1 + (p-1)*k:p*n);

		% else we (hopefully) correct the error
		else
			% Find which coset leader corresponds to the syndrome
			index = 0;
			for o = 1:n
				if( syndromes(o,:) == syn )	
					index = o;
				end
			end
			y((p-1)*n+1:p*n) = mod(y((p-1)*n+1:p*n) + coset_leaders(index,:),2);
			x((p-1)*k+1:p*k) = y(p*m+1 + (p-1)*k:p*n);
		end
	end
