function y = hamming_encoder(x, G)
% Encodes the bitstream x using Hamming coding with the given generator matrix G
	
	% Get input and output sizes
	k = size(G,1);
	n = size(G,2);

	% The length of the input vector must be a multiple of k
	if mod(length(x),k) ~= 0
		error('Length of input must be a multiple of k.', 'x_error')
	end
	
	% Transpose the array if necessary
	if size(x,1) ~= 1
		x = x';
	end

	% Calculate the number of loops required for encoding
	loops = ceil(length(x)/k);

	% Initialize the output vector
	y = [];

	% Encode
	for l = 1:loops
		y = [ y mod(x((l-1)*k+1:l*k)*G,2) ];
	end
