function [G, H, std_array]= hamming_matrices(m)
% Creates the generator matrix G, the parity check matrix H and a matrix containing
% the standard array coset leaders and the corresponding syndromes for a Hamming code 
% with m parity bits.

	if m < 2 
		error('Argument m must be greater than 1.', 'm_error')
	end

	% Codeword length
	n = 2^m - 1;

	% Number of information symbols per codeword
	k = 2^m - m - 1;

	% We create the parity check matrix H (recall that m = n-k)
	m_tuples = (dec2bin(1:2^m-1) - 48)';
	P_trans = m_tuples( :,sum(m_tuples) ~= 1 );
	H = [ eye(m) P_trans ];

	% From H, we create the generator matrix G
	G = [ P_trans' eye(k) ];

	% We create the matrix containing the standard array coset leaders and the corresponding syndromes
	std_array = [ eye(n) eye(n)*H' ];
	
